/*
 * desproxy.c: Main program file for desproxy
 *
 * Copyright (C) 2002 Miguelanxo Otero Salgueiro
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the tems of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 */

#include "desproxy.h"
#define PROGRAM_NAME "desproxy"
#define PROGRAM_VERSION VERSION

int request_sock;
int connstate[MAX_CONNECTIONS];
char *remote_host,*remote_port;

/*
 * Function: void EOC(int connection)
 * Purpose : terminates connection
 * Params  : int connection - connection numbre to terminate
 */
void EOC(int connection)
{
	debug_printf(">EOC(%d)\n",connection);
	debug_printf("connstate[%d]=%d\n",connection,connstate[connection]);
	client_sock_is_free[connection]=1;
	FD_CLR(client_sock[connection],&mask);
	FD_CLR(client_sock[connection],&rmask);
	close(client_sock[connection]);
	if (connstate[connection] == BICONNECTED)
	{
 		debug_printf("connstate[connection]==BICONNECTED\n");
		FD_CLR(proxy_sock[connection],&mask);
		FD_CLR(proxy_sock[connection],&rmask);
		close(proxy_sock[connection]);
	}
	connstate[connection]=TO_RESET;
	print_connection(connection,gettext ("End of connection\n"));
	debug_printf("EOC>\n");
}

void client_to_proxy(int connection)
{
	int count;

	print_connection(connection,"client -> proxy");
	if ((count=read(client_sock[connection],buffer,sizeof(buffer)))==-1)
	{
		perror("read");
		EOC(connection);
		return;
	}
	printf(" %4d bytes read\n",count);
	if (count==0)
	{
		EOC(connection);
		return;
	} else {
		if ((count=write(proxy_sock[connection],buffer,count))==-1)
		{
			perror("write");
			EOC(connection);
		}
	}
}

void proxy_to_client(int connection)
{
	int count;

	print_connection(connection,"proxy -> client");
	if ((count=read(proxy_sock[connection],buffer,sizeof(buffer)))==-1)
	{
		perror("read");
		EOC(connection);
		return;
	}
	printf(" %4d bytes read\n",count);
	if (count==0)
	{
		EOC(connection);
		return;
	} else {
		if ((count=write(client_sock[connection],buffer,count))==-1)
		{
			perror("write");
			EOC(connection);
		}
	}
}

void parse_command_line(int argc, char **argv)
{
        debug_printf(">parse_command_line(COMMAND_LINE)\n");
	if (argc !=6)
	{
		printf(gettext ("Usage: desproxy remote_host remote_port proxy_host proxy_port local_port\n\n"));
        	exit(1);
        }
	remote_host=argv[1];
	remote_port=argv[2];
	proxy_host=argv[3];
	proxy_port=argv[4];
	local_port=argv[5];
	debug_printf("parse_command_line>\n");
}

void process_connection_request(void)
{
	int connection;

        debug_printf(">process_connection_request()\n");
	for(connection=0;connection<MAX_CONNECTIONS;connection++) if (client_sock_is_free[connection]) break;
	if (connection == MAX_CONNECTIONS)
	{
		printf(gettext ("There are no connections available at this time\n"));
	} else {
		client_length=sizeof(client);
		client_sock[connection]=accept(request_sock,(struct sockaddr *)&client,&client_length);
		if (client_sock[connection] < 0)
		{
			perror("accept");
			EOC(connection);
			return;
		}
		client_sock_is_free[connection]=0;
		printf(gettext ("Connection request from %s, port %d\n"),inet_ntoa(client.sin_addr),ntohs(client.sin_port));
		if (client_sock[connection] >maxfd) maxfd=client_sock[connection];
		FD_SET(client_sock[connection],&mask);
		printf(gettext ("Connecting to http proxy (%s:%s)\n"),proxy_host,proxy_port);
		if (connect_host_to_proxy(connection,remote_host,remote_port)!=0)
		{
			printf("connect_host_to_proxy: ERROR\n");
			EOC(connection);
			return;
		} else {
			connstate[connection]=BICONNECTED;
		}
	}
	debug_printf("process_connection_request>\n");
}

int main (int argc, char **argv) {
	
	int connection,nfound;
	struct timeval timeout;

	signal(SIGPIPE,(sig_t)PIPE); // captures SIGPIPE
	initialize_gettext();
	print_program_version(PROGRAM_NAME,PROGRAM_VERSION);
	parse_command_line(argc,argv);	

	request_sock=listen_in_TCP_port(atoi(local_port));
	printf(gettext ("Press ENTER to Quit\n"));
	mark_all_client_socks_as_free();
	
	FD_ZERO(&mask);
	FD_SET(request_sock,&mask);
	FD_SET(0,&mask);
	maxfd=request_sock;
	while (1)
	{
		rmask=mask;
		timeout.tv_sec=5;
		timeout.tv_usec=0;
		nfound=select(maxfd+1,&rmask,NULL,NULL, &timeout);
		debug_printf("%d",nfound);
		if (nfound<0)
		{
			if (errno==EINTR)
			{
				printf(gettext ("Interrupted by system call\n"));
				continue;
			}
			perror("select");
			exit(1);
		}
		
		if (FD_ISSET(0,&rmask)) // if ENTER then exit normally
		{
			close(request_sock);
			exit(0);
		}

		if (FD_ISSET(request_sock, &rmask))
		{
			process_connection_request();
		}

		for (connection=0;connection<MAX_CONNECTIONS;connection++)
		{
			if (!client_sock_is_free[connection])
			{
				debug_printf("client_sock[%d]=%d\n",connection,client_sock[connection]);
				debug_printf("connstate[%d]=%d\n",connection,connstate[connection]);
				if (FD_ISSET(client_sock[connection], &rmask))
				{
					client_to_proxy(connection);
				}
				if (connstate[connection]==BICONNECTED)
				{
					if (FD_ISSET(proxy_sock[connection], &rmask))
					{
						proxy_to_client(connection);
					}
				}
			}
		}
		fflush(stdout);
	}
}
