// socket2socket.c
//

#include <signal.h>
#include "desproxy.h"
#define MAX_CONNECTIONS 10
#define PROGRAM_VERSION VERSION
#define PROGRAM_NAME "socket2socket"

unsigned char client_sock_is_free[MAX_CONNECTIONS];
	int client_sock[MAX_CONNECTIONS];
	int remote_sock[MAX_CONNECTIONS];

void EOC(int index)
{

	close(client_sock[index]);
        close(remote_sock[index]);
	FD_CLR(client_sock[index], &mask);
	FD_CLR(remote_sock[index], &mask);
	client_sock_is_free[index]=1;
	printf(gettext ("End of connection\n"));
	fflush(stdout);
	fflush(stderr);
}

int main (int argc, char **argv) {
	
	int request_sock,addrlen;
	int index,count;
        struct sockaddr_in server;
	struct sockaddr_in remote;
	struct hostent *remote_hostent;
	char *remote_host;
	char *remote_port;
	char *local_port;
	int nfound,maxfd;
	struct timeval timeout;
	int outfile=1;
	char *outfilename;
	
	signal(SIGPIPE,(sig_t)PIPE);
	print_program_version(PROGRAM_NAME,PROGRAM_VERSION);
	
	if ((argc <4)||(argc>5)) {
		printf(gettext ("Usage: socket2socket remote_host remote_port local_port [arquivo_saida]\n\n"));
        	exit(1);
        }
	remote_host=argv[1];
	remote_port=argv[2];
	local_port=argv[3];
	
	if (argc==5)
	{
		outfilename=argv[4];
		if ((outfile=open(outfilename,O_CREAT|O_WRONLY,S_IREAD|S_IWRITE))==-1)
		{
			perror("open");
			exit(1);
		}
	}
	if ((request_sock=socket(AF_INET,SOCK_STREAM,0))<0)
	{
		perror("socket");
		exit(1);
	}
	memset(&server,0,sizeof server);
	server.sin_family=AF_INET;
	server.sin_addr.s_addr=INADDR_ANY;
	server.sin_port=htons(atoi(local_port));
	if ((bind(request_sock,(struct sockaddr *)&server,sizeof server)) < 0)
	{
		perror("bind");
		exit(1);
	}
	if (listen(request_sock,SOMAXCONN)<0)
	{
		perror("listen");
		exit(1);
	}
	printf(gettext ("Listening in port %d\n"),atoi(local_port));
	printf(gettext ("Press ENTER to Quit\n"));
	
	mark_all_client_socks_as_free();
	FD_ZERO(&mask);
	FD_SET(request_sock,&mask);
	FD_SET(0,&mask);
	maxfd=request_sock;
	while (1)
	{
		rmask=mask;
		timeout.tv_sec=5;
		timeout.tv_usec=0;
		nfound=select(maxfd+1,&rmask,NULL,NULL, &timeout);
		if (nfound<0)
		{
			if (errno==EINTR)
			{
				printf(gettext ("Interrupted by system call\n"));
				continue;
			}
			perror("select");
			exit(1);
		}
		if (FD_ISSET(0,&rmask))
		{
			close(request_sock);
			exit(0);
		}
		if (FD_ISSET(request_sock, &rmask))
		{
			addrlen=sizeof(remote);
			for(index=0;index<MAX_CONNECTIONS;index++) if (client_sock_is_free[index]) break;
			if (index == MAX_CONNECTIONS)
			{
				printf(gettext ("There are no connections available at this time\n"));
			} else {
				client_sock_is_free[index]=0;
				client_sock[index]=accept(request_sock,(struct sockaddr *)&remote,&addrlen);
				if (client_sock[index] < 0)
				{
					perror("accept");
					exit(1);
				}
				printf(gettext ("Connection request from %s, port %d\n"),inet_ntoa(remote.sin_addr),ntohs(remote.sin_port));
				if (client_sock[index] >maxfd) maxfd=client_sock[index];
				printf(gettext ("Connecting to remote host (%s:%s)\n"),remote_host,remote_port);
				if ((remote_sock[index]=socket(AF_INET,SOCK_STREAM,0))<0)
				{
					perror("socket");
					exit(1);
				}
				if (remote_sock[index] >maxfd) maxfd=remote_sock[index];
				if ((remote_hostent=gethostbyname(remote_host))==0)
				{
					perror("gethostbyname");
					exit(1);
				}
				memset(&remote,0,sizeof remote);
				remote.sin_family=AF_INET;
				memcpy(&remote.sin_addr,remote_hostent->h_addr,remote_hostent->h_length);
				remote.sin_port=htons(atoi(remote_port));
				if (connect(remote_sock[index],(struct sockaddr *)&remote,sizeof remote) < 0)
				{
					close(remote_sock[index]);
					printf("\n");
					perror("connect");
					exit(1);
				}
				printf(gettext ("Bidirectional connection stablished\n"));
				printf("(%s:%s) <-> (localhost)\n",remote_host,remote_port);
				FD_SET(remote_sock[index],&mask);
				FD_SET(client_sock[index],&mask);
			}
		}
		for (index=0;index<MAX_CONNECTIONS;index++)
		{
			if (!client_sock_is_free[index])
			{
				if (FD_ISSET(remote_sock[index], &rmask))
				{
	 	 			printf("remote -> local (%d) ",index);fflush(stdout); 
					if ((count=read(remote_sock[index],buffer,sizeof(buffer)))==-1)
					{
						perror("read");
						EOC(index);
					} else {
						if (count==0)
						{
							EOC(index);
						} else {
							if ((count=write(client_sock[index],buffer,count))==-1)
							{
								perror("\nwrite");fflush(stderr);
								EOC(index);
							} else {
								if (argc==5)
								{
									write(outfile,"[R->L]",6);
									write(outfile,buffer,count);
									fsync(outfile);
								}
								printf(" %d Bytes\n",count);
								fflush(stdout);
							}
						}
					}
				}
			}
			if (!client_sock_is_free[index])
			{
				if (FD_ISSET(client_sock[index], &rmask))
				{
		 			printf("local -> remote (%d)",index);fflush(stdout); 
					if ((count=read(client_sock[index],buffer,sizeof(buffer)))==-1)
					{
						perror("read");
						EOC(index);
					} else {
						if (count==0)
						{
							EOC(index);
						} else { 
							if ((count=write(remote_sock[index],buffer,count))==-1)
							{ 
								perror("write"); 
								EOC(index); 
							} else {
								if (argc==5)
								{
									write(outfile,"[L->R]",6);
									write(outfile,buffer,count);
									fsync(outfile);
								}
								printf(" %d bytes\n",count);
								fflush(stdout);
							}
						}
					}
				}
			}
		}
		fflush(stdout);
		fflush(stderr);
	}
	exit(0);
}
